from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import scoped_session, sessionmaker
from zope.sqlalchemy import ZopeTransactionExtension
from sqlalchemy import Column, String, Integer, UniqueConstraint


DBSession = scoped_session(sessionmaker(extension=ZopeTransactionExtension()))  # initialise a ORM session, ought to be reused across the different modules
Base = declarative_base()  # initialise a declarative Base instance to use for the web app models


class ServiceFunctionEndpoint(Base):
    """
    This class defines the main model of the WHOAMI API, declaring the global tags for a specific service function on a specific endpoint.
    """

    __tablename__ = 'sfendpoints'  # table name in the PostgreSQL database

    __table_args__ = (UniqueConstraint('sf_i', 'sf_endpoint', 'sr'),)  # defines a unique constraint across 3 columns - sf_i, sf_endpoint, sr

    uid = Column(Integer, primary_key=True, autoincrement=True)  # a primary key integer field (auto incremented)

    location = Column(String)  # cluster label
    sfc = Column(String)  # service function chain label
    sfc_i = Column(String)  # service function chain instance identifier
    sf = Column(String)  # service function label
    sf_i = Column(String)   # service function identifier (potentially FQDN)
    sf_endpoint = Column(String)  # service function endpoint (potentially IP address)
    sr = Column(String)  # service router ID - service router that connects the VM to FLAME
